#pragma once

#include "libAM50288H.h"

namespace Tnksoft {
namespace AM50288H {
	public enum class Mark
	{
		Asterisk = 57,
		Diamond = 58,
		Square = 59,
		Anntena = 60,
		Battery = 61,
		Next = 136,
		Bell = 137,
		Cycle = 138,
		Note = 140,
		Speaker = 141,
	};

	[Platform::Metadata::Flags]
	public enum class SegmentPosition : unsigned int
	{
		None = 0,
		Top = 1,
		RightTop = 2,
		RightBottom = 4,
		Bottom = 8,
		LeftBottom = 16,
		LeftTop = 32,
		MiddleLeft = 64,
		MiddleRight = 128,
		TopLeft = 256,
		TopCenter = 512,
		TopRight = 1024,
		BottomRight = 2048,
		BottomCenter = 4096,
		BottomLeft = 8192,

		A = Top,
		B = RightTop,
		C = RightBottom,
		D = Bottom,
		E = LeftBottom,
		F = LeftTop,
		G1 = MiddleLeft,
		G2 = MiddleRight,
		J = TopLeft,
		K = TopCenter,
		L = TopRight,
		M = BottomRight,
		N = BottomCenter,
		P = BottomLeft,
	};

	public ref class AM50288H sealed
	{
	private:
		::AM50288H _am;
	public:
		AM50288H();
		void begin(byte pinLE, byte pinDIN, byte pinSCK);
		void update();
		void clear();

		void setSegmentAsBits(int pos, unsigned short segbits);
		void setSegment(int pos, SegmentPosition spos);

		unsigned short getAsciiSegment(wchar_t ascii);
		SegmentPosition getSegmentFlag(unsigned short seg);
		void setAscii(int pos, wchar_t ascii);
		void setAsciiString(Platform::String^ str);
		void setMark(Mark mark, bool set);

		void shiftSegments(bool left);
		void shiftSegmentsLeft();
		void shiftSegmentsRight();
	};
}
}
