#include "libAM50288H.h"

AM50288H::AM50288H()
{
}

AM50288H::AM50288H(byte pinLE, byte pinDIN, byte pinSCK)
{
	setPins(pinLE, pinDIN, pinSCK);
}

AM50288H::~AM50288H()
{
}

void AM50288H::setPins(byte pinLE, byte pinDIN, byte pinSCK)
{
	ple = pinLE;
	pdin = pinDIN;
	psck = pinSCK;

	pinMode(ple, OUTPUT);
	pinMode(pdin, OUTPUT);
	pinMode(psck, OUTPUT);
}

void AM50288H::begin()
{
	clear();
}

void AM50288H::update()
{
	if (psck == 0xFF) return;

	for (int i = DinsLen - 1; i >= 0; i--) {
		digitalWrite(psck, LOW);
		digitalWrite(pdin, dins[i]);
		delayMicroseconds(1);
		digitalWrite(psck, HIGH);
		delayMicroseconds(1);
	}

	delayMicroseconds(1);
	digitalWrite(ple, HIGH);
	delayMicroseconds(1);
	digitalWrite(ple, LOW);
}

void AM50288H::clear()
{
	memset(dins, 0, DinsLen);
	update();
}

void AM50288H::setSegmentAsBits(int pos, unsigned short segbits)
{
	for (int i = 0; i < Segments; i++) {
		dins[(int)segarray[pos][i]] = ((segbits >> i) & 0x1);
	}
}

void AM50288H::setSegment(int pos, AM50288H_SegmentPosition spos)
{
	setSegmentAsBits(pos, (unsigned short)spos);
}

unsigned short AM50288H::getAsciiSegment(char ascii)
{
	unsigned short sb;
	switch (ascii) {
	case '0': sb = 0x243F; break;
	case '1': sb = 0x0006; break;
	case '2': sb = 0x00DB; break;
	case '3': sb = 0x008F; break;
	case '4': sb = 0x00E6; break;
	case '5': sb = 0x0869; break;
	case '6': sb = 0x00FD; break;
	case '7': sb = 0x0007; break;
	case '8': sb = 0x00FF; break;
	case '9': sb = 0x00EF; break;
	case 'A': sb = 0x00F7; break;
	case 'B': sb = 0x128F; break;
	case 'C': sb = 0x0039; break;
	case 'D': sb = 0x120F; break;
	case 'E': sb = 0x00F9; break;
	case 'F': sb = 0x0071; break;
	case 'G': sb = 0x00BD; break;
	case 'H': sb = 0x00F6; break;
	case 'I': sb = 0x1200; break;
	case 'J': sb = 0x001E; break;
	case 'K': sb = 0x0C70; break;
	case 'L': sb = 0x0038; break;
	case 'M': sb = 0x0536; break;
	case 'N': sb = 0x0936; break;
	case 'O': sb = 0x003F; break;
	case 'P': sb = 0x00F3; break;
	case 'Q': sb = 0x083F; break;
	case 'R': sb = 0x08F3; break;
	case 'S': sb = 0x00ED; break;
	case 'T': sb = 0x1201; break;
	case 'U': sb = 0x003E; break;
	case 'V': sb = 0x2430; break;
	case 'W': sb = 0x2836; break;
	case 'X': sb = 0x2D00; break;
	case 'Y': sb = 0x1500; break;
	case 'Z': sb = 0x2409; break;
	case 'a': sb = 0x1058; break;
	case 'b': sb = 0x0878; break;
	case 'c': sb = 0x00D8; break;
	case 'd': sb = 0x208E; break;
	case 'e': sb = 0x2058; break;
	case 'f': sb = 0x0071; break;
	case 'g': sb = 0x048E; break;
	case 'h': sb = 0x1070; break;
	case 'i': sb = 0x1000; break;
	case 'j': sb = 0x000E; break;
	case 'k': sb = 0x1E00; break;
	case 'l': sb = 0x0030; break;
	case 'm': sb = 0x10D4; break;
	case 'n': sb = 0x1050; break;
	case 'o': sb = 0x00DC; break;
	case 'p': sb = 0x0170; break;
	case 'q': sb = 0x0486; break;
	case 'r': sb = 0x0050; break;
	case 's': sb = 0x0888; break;
	case 't': sb = 0x0078; break;
	case 'u': sb = 0x001C; break;
	case 'v': sb = 0x0804; break;
	case 'w': sb = 0x2814; break;
	case 'x': sb = 0x28C0; break;
	case 'y': sb = 0x080C; break;
	case 'z': sb = 0x2048; break;
	case '!': sb = 0x0006; break;
	case '\"': sb = 0x0220; break;
	case '#': sb = 0x12CE; break;
	case '$': sb = 0x12ED; break;
	case '%': sb = 0x2424; break;
	case '&': sb = 0x0B5D; break;
	case '\'': sb = 0x0400; break;
	case '(': sb = 0x0C00; break;
	case ')': sb = 0x2100; break;
	case '*': sb = 0x3FC0; break;
	case '+': sb = 0x12C0; break;
	case ',': sb = 0x2000; break;
	case '-': sb = 0x00C0; break;
	case '.': sb = 0x0000; break;
	case '/': sb = 0x2400; break;
	case ':': sb = 0x1200; break;
	case ';': sb = 0x2200; break;
	case '<': sb = 0x0C00; break;
	case '=': sb = 0x00C8; break;
	case '>': sb = 0x2100; break;
	case '?': sb = 0x1083; break;
	case '@': sb = 0x02BB; break;
	case '[': sb = 0x0039; break;
	case '\\': sb = 0x0900; break;
	case ']': sb = 0x000F; break;
	case '^': sb = 0x2403; break;
	case '_': sb = 0x0008; break;
	case '`': sb = 0x0100; break;
	case '{': sb = 0x2149; break;
	case '|': sb = 0x1200; break;
	case '}': sb = 0x0C89; break;
	case '~': sb = 0x0520; break;
	default: sb = 0x0000; break;
	}
	return sb;
}

AM50288H_SegmentPosition AM50288H::getSegmentFlag(unsigned short seg)
{
	unsigned short res = 0;
	for (int i = 0; i < Segments; i++) {
		if ((seg >> i & 0x1) != 0) res |= (unsigned short)(0x1 << i);
	}
	return (AM50288H_SegmentPosition)res;
}

void AM50288H::setAscii(int pos, char ascii)
{
	setSegmentAsBits(pos, getAsciiSegment(ascii));
}

void AM50288H::setAsciiString(const char *str)
{
	int slen = strlen(str);
	if (slen > 8) slen = 8;
	for (int i = 0; i < 8; i++) {
		if (i < slen) {
			setAscii(i, str[i]);
		} else {
			setSegment(i, AM50288H_SegmentPosition::None);
		}
	}
}

void AM50288H::shiftSegmentsLeft()
{
	for (int i = 0; i < Digits - 1; i++) {
		for (int j = 0; j < Segments; j++) {
			dins[(int)segarray[i][j]] = dins[(int)segarray[i + 1][j]];
		}
	}
	for (int j = 0; j < Segments; j++) dins[(int)segarray[Digits - 1][j]] = LOW;
}

void AM50288H::shiftSegmentsRight()
{
	for (int i = Digits - 1; i >= 1; i--) {
		for (int j = 0; j < Segments; j++) {
			dins[(int)segarray[i][j]] = dins[(int)segarray[i - 1][j]];
		}
	}
	for (int j = 0; j < Segments; j++) dins[(int)segarray[0][j]] = LOW;
}

void AM50288H::shiftSegments(bool left)
{
	if (left == true) {
		shiftSegmentsLeft();
	} else {
		shiftSegmentsRight();
	}
}

void AM50288H::setMark(AM50288H_Mark mark, bool set)
{
	dins[(int)mark] = (set == true) ? HIGH : LOW;
}
