#ifndef __AM50288H_H__
#define __AM50288H_H__

#include <arduino.h>

enum AM50288H_Mark
{
	Asterisk = 57,
	Diamond = 58,
	Square = 59,
	Anntena = 60,
	Battery = 61,
	Next = 136,
	Bell = 137,
	Cycle = 138,
	Note = 140,
	Speaker = 141,
};

enum AM50288H_SegmentPosition
{
	None = 0,
	Top = 1,
	RightTop = 2,
	RightBottom = 4,
	Bottom = 8,
	LeftBottom = 16,
	LeftTop = 32,
	MiddleLeft = 64,
	MiddleRight = 128,
	TopLeft = 256,
	TopCenter = 512,
	TopRight = 1024,
	BottomRight = 2048,
	BottomCenter = 4096,
	BottomLeft = 8192,

	A = Top,
	B = RightTop,
	C = RightBottom,
	D = Bottom,
	E = LeftBottom,
	F = LeftTop,
	G1 = MiddleLeft,
	G2 = MiddleRight,
	J = TopLeft,
	K = TopCenter,
	L = TopRight,
	M = BottomRight,
	N = BottomCenter,
	P = BottomLeft,
};

class AM50288H
{
private:
	enum SegmentBit
	{
		SGB_1J = 0, SGB_1G2 = 1, SGB_1P = 2, SGB_1N = 3, SGB_1M = 4,
		SGB_1L = 5, SGB_1K = 6, SGB_2J = 7, SGB_2G2 = 8, SGB_2P = 9,
		SGB_2N = 10, SGB_2M = 11, SGB_2L = 12, SGB_2K = 13, SGB_3J = 14,
		SGB_3G2 = 15, SGB_3P = 16, SGB_3N = 17, SGB_3M = 18, SGB_3L = 19,
		SGB_3K = 20, SGB_4J = 21, SGB_4G2 = 22, SGB_4P = 23, SGB_4N = 24,
		SGB_4M = 25, SGB_4L = 26, SGB_4K = 27, SGB_5J = 28, SGB_5G2 = 29,
		SGB_5P = 30, SGB_5N = 31, SGB_5M = 32, SGB_5L = 33, SGB_5K = 34,
		SGB_6J = 35, SGB_6G2 = 36, SGB_6P = 37, SGB_6N = 38, SGB_6M = 39,
		SGB_6L = 40, SGB_6K = 41, SGB_7J = 42, SGB_7G2 = 43, SGB_7P = 44,
		SGB_7N = 45, SGB_7M = 46, SGB_7L = 47, SGB_7K = 48, SGB_8J = 49,
		SGB_8G2 = 50, SGB_8P = 51, SGB_8N = 52, SGB_8M = 53, SGB_8L = 54,
		SGB_8K = 55, SGB_1F = 80, SGB_1G1 = 81, SGB_1E = 82, SGB_1D = 83,
		SGB_1C = 84, SGB_1B = 85, SGB_1A = 86, SGB_2F = 87, SGB_2G1 = 88,
		SGB_2E = 89, SGB_2D = 90, SGB_2C = 91, SGB_2B = 92, SGB_2A = 93,
		SGB_3F = 94, SGB_3G1 = 95, SGB_3E = 96, SGB_3D = 97, SGB_3C = 98,
		SGB_3B = 99, SGB_3A = 100, SGB_4F = 101, SGB_4G1 = 102, SGB_4E = 103,
		SGB_4D = 104, SGB_4C = 105, SGB_4B = 106, SGB_4A = 107, SGB_5F = 108,
		SGB_5G1 = 109, SGB_5E = 110, SGB_5D = 111, SGB_5C = 112, SGB_5B = 113,
		SGB_5A = 114, SGB_6F = 115, SGB_6G1 = 116, SGB_6E = 117, SGB_6D = 118,
		SGB_6C = 119, SGB_6B = 120, SGB_6A = 121, SGB_7F = 122, SGB_7G1 = 123,
		SGB_7E = 124, SGB_7D = 125, SGB_7C = 126, SGB_7B = 127, SGB_7A = 128,
		SGB_8F = 129, SGB_8G1 = 130, SGB_8E = 131, SGB_8D = 132, SGB_8C = 133,
		SGB_8B = 134, SGB_8A = 135,
	};

	const static int Digits = 8;
	const static int Segments = 14;
	const static int DinsLen = 142;
	const SegmentBit segarray[Digits][Segments] = {
		{ SGB_1A, SGB_1B, SGB_1C, SGB_1D, SGB_1E, SGB_1F, SGB_1G1, SGB_1G2, SGB_1J, SGB_1K, SGB_1L, SGB_1M, SGB_1N, SGB_1P },
		{ SGB_2A, SGB_2B, SGB_2C, SGB_2D, SGB_2E, SGB_2F, SGB_2G1, SGB_2G2, SGB_2J, SGB_2K, SGB_2L, SGB_2M, SGB_2N, SGB_2P },
		{ SGB_3A, SGB_3B, SGB_3C, SGB_3D, SGB_3E, SGB_3F, SGB_3G1, SGB_3G2, SGB_3J, SGB_3K, SGB_3L, SGB_3M, SGB_3N, SGB_3P },
		{ SGB_4A, SGB_4B, SGB_4C, SGB_4D, SGB_4E, SGB_4F, SGB_4G1, SGB_4G2, SGB_4J, SGB_4K, SGB_4L, SGB_4M, SGB_4N, SGB_4P },
		{ SGB_5A, SGB_5B, SGB_5C, SGB_5D, SGB_5E, SGB_5F, SGB_5G1, SGB_5G2, SGB_5J, SGB_5K, SGB_5L, SGB_5M, SGB_5N, SGB_5P },
		{ SGB_6A, SGB_6B, SGB_6C, SGB_6D, SGB_6E, SGB_6F, SGB_6G1, SGB_6G2, SGB_6J, SGB_6K, SGB_6L, SGB_6M, SGB_6N, SGB_6P },
		{ SGB_7A, SGB_7B, SGB_7C, SGB_7D, SGB_7E, SGB_7F, SGB_7G1, SGB_7G2, SGB_7J, SGB_7K, SGB_7L, SGB_7M, SGB_7N, SGB_7P },
		{ SGB_8A, SGB_8B, SGB_8C, SGB_8D, SGB_8E, SGB_8F, SGB_8G1, SGB_8G2, SGB_8J, SGB_8K, SGB_8L, SGB_8M, SGB_8N, SGB_8P }
	};

	byte ple, pdin, psck = 0xFF;
	byte dins[DinsLen];

public:
	AM50288H();
	AM50288H(byte pinLE, byte pinDIN, byte pinSCK);
	~AM50288H();

	void setPins(byte pinLE, byte pinDIN, byte pinSCK);
	void begin();
	void update();
	void clear();

	void setSegmentAsBits(int pos, unsigned short segbits);
	void setSegment(int pos, AM50288H_SegmentPosition spos);

	unsigned short getAsciiSegment(char ascii);
	AM50288H_SegmentPosition getSegmentFlag(unsigned short seg);
	void setAscii(int pos, char ascii);
	void setAsciiString(const char *str);
	void setMark(AM50288H_Mark mark, bool set);

	void shiftSegments(bool left);
	void shiftSegmentsLeft();
	void shiftSegmentsRight();
};

#endif __AM50288H_H__
