﻿using Windows.UI.Xaml.Controls;
using Tnksoft.AM50288H;
using Windows.UI.Xaml;

namespace AM50288HDemo
{
	public sealed partial class MainPage : Page
	{
		private SegmentSwitch[] ss;
		private CheckBox[] cb;
		private AM50288H amh;

		public MainPage()
		{
			this.InitializeComponent();

			ss = new SegmentSwitch[] { s1, s2, s3, s4, s5, s6, s7, s8 };
			cb = new CheckBox[] { chkAstarisk, chkDiamond, chkSquare,
				chkBattery, chkNext, chkBell, chkCycle, chkNote, chkSpeaker };

			for (int i = 1; i <= 8; i++) {
				cmbCharPos.Items.Add(i.ToString());
			}
			cmbCharPos.SelectedIndex = 0;

			amh = new AM50288H();

			// GPIO5 = 29, GPIO6 = 31, GPIO13 = 33 on Raspberry Pi
			amh.begin(29, 31, 33);
		}

		private void OnClickSetChar(object sender, RoutedEventArgs e)
		{
			if (txtChar.Text.Length == 0) return;

			int index = cmbCharPos.SelectedIndex;
			char c = txtChar.Text[0];
			ushort seg = amh.getAsciiSegment(c);
			if (seg == 0x0000) return;

			//amh.setAscii(index, c);

			SegmentPosition sp = amh.getSegmentFlag(seg);
			ss[index].SetSegments(sp);

			index++;
			if (index >= 8) index = 0;
			cmbCharPos.SelectedIndex = index;
		}

		private void OnClickSetText(object sender, RoutedEventArgs e)
		{
			string txt = txtString.Text;

			//amh.setAsciiString(txt);

			int tlen = txt.Length;
			for (int i = 0; i < 8; i++) {
				ushort n = (i < tlen) ? amh.getAsciiSegment(txt[i]) : (ushort)0x0000;
				ss[i].SetSegments(amh.getSegmentFlag(n));
			}
		}

		private void OnMarkCheckChanged(object sender, RoutedEventArgs e)
		{
			foreach (var c in cb) {
				if (c == sender) {
					string name = c.Name.Substring(3);
					Mark amm = (Mark)System.Enum.Parse(typeof(Mark), name);
					amh.setMark(amm, c.IsChecked ?? false);
					break;
				}
			}
		}

		private void OnClickUpdate(object sender, RoutedEventArgs e)
		{
			for (int i = 0; i < 8; i++) {
				amh.setSegment(i, ss[i].GetSegments());
			}

			amh.update();
		}

		private void OnClickShiftLeft(object sender, RoutedEventArgs e)
		{
			for (int i = 0; i < 7; i++) {
				ss[i].SetSegments(ss[i + 1].GetSegments());
			}
			ss[7].SetSegments(SegmentPosition.None);

			amh.shiftSegmentsLeft();
			amh.update();
		}

		private void OnClickShiftRight(object sender, RoutedEventArgs e)
		{
			for (int i = 7; i >= 1; i--) {
				ss[i].SetSegments(ss[i - 1].GetSegments());
			}
			ss[0].SetSegments(SegmentPosition.None);

			amh.shiftSegmentsRight();
			amh.update();
		}

		private void OnClickClear(object sender, RoutedEventArgs e)
		{
			foreach (var s in ss) s.SetSegments(SegmentPosition.None);
			amh.clear();
		}
	}
}
